#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "stdio.h"
#include "math.h"
#include "stdlib.h"
#include "string.h"

#include "debug.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "Symbol.h"
#include "2DCMC.h"
#include "auxiliar.h"
#include "GLTile.h"
#include "2DCMC.h"
#include "keyboardstate.h"
#include "randomc.h"
#include "VirtualController.h"

#include "Binterface.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "MusicManager.h"
#include "KObject.h"
#include "KPlayer.h"
#include "Knightmare.h"
#include "KnightmareApp.h"

// #include "font_extractor.h"


int KnightmareApp::game_cycle(KEYBOARDSTATE *k)
{
	if (k->key_press(m_keys_configuration1[KKEY_QUIT])) {
		if (m_game_paused_state == 0) {
			// if you are playing, pause:
			m_game_paused_state = 1;
			m_game_paused_item_selected = 0;			
		} else {
			// if you are in the process of pausing or unpausing, then just reverse:
			m_game_paused_state = -m_game_paused_state;
/*
			if (m_game_paused_state > 0) {
				if (m_game_paused_item_selected == 0) {
					m_game_paused_state = -m_game_paused_state;
				} else {
					// QUIT:
					m_MusicM->music_fadeout(0,50);
					return STATE_GAMEOVER;					
				}
			} else {
				m_game_paused_item_selected = 0;
				m_game_paused_state = -m_game_paused_state;
			}
 */
		}
		
		m_SFXM->SFX_play("sfx/pause", m_sfx_volume);
		
	}

	if (m_state_cycle==0) {
		m_MusicM->music_play(0, "level1", m_music_volume,-1);
	}
	
	// pause:
	{
		// set players to sleep:
		if (m_game_paused_state>0) {
			// set all characters in sleep mode:
			List<KObject> l;
			KObject *character;
			l.Instance(*(m_game->get_map()->getCharacters()));
			while(l.Iterate(character)) {
				character->set_current_animation(character->get_state_animation(PLAYER_STATE_PAUSE));
				character->set_last_tile_used(character->get_current_animation()->cycle());
			}
			
		}			
		
		if (m_game_paused_state==0 && m_game_paused_item_selected==1) {
			// player selected quit:
			m_MusicM->music_fadeout(0,50);
			return STATE_GAMEOVER;					
		}
		if (m_game_paused_state>0) {
			if (k->key_press(m_keys_configuration1[KKEY_UP]) ||
				k->key_press(m_keys_configuration1[KKEY_DOWN]) ||
				k->key_press(m_keys_configuration2[KKEY_UP]) ||
				k->key_press(m_keys_configuration2[KKEY_DOWN])) {
				m_game_paused_item_selected = 1 - m_game_paused_item_selected;
			}
		}
		
		if (m_game_paused_state!=0) {
			if (m_game_paused_state>0 && m_game_paused_state<25) m_game_paused_state++;
			if (m_game_paused_state<0) m_game_paused_state++;
		}
		
		// set volume according to pause (in pause, volume is only 10%)
		float f = 0;
		if (m_game_paused_state>0) {
			f = m_game_paused_state/25.0f;
		} 
		if (m_game_paused_state<0) {
			f = m_game_paused_state/-25.0f;
		}
		f = 1 - f*0.9;
		m_MusicM->music_volume(0,m_music_volume*f);
	}

	if (m_game_paused_state<=0) {
		VirtualController *vc;
		m_vc_l.Rewind();
		while(m_vc_l.Iterate(vc)) {
			vc->new_cycle();
			vc->m_keyboardstate = k;
			if (vc->m_ID==0) {
				if (k->key_pressed(m_keys_configuration1[KKEY_UP])) vc->m_joystick[VC_UP]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_DOWN])) vc->m_joystick[VC_DOWN]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_LEFT])) vc->m_joystick[VC_LEFT]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_RIGHT])) vc->m_joystick[VC_RIGHT]=true;
				if (k->key_pressed(m_keys_configuration1[KKEY_FIRE])) vc->m_button[0]=true;
			}
			if (vc->m_ID==1) {
				if (k->key_pressed(m_keys_configuration2[KKEY_UP])) vc->m_joystick[VC_UP]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_DOWN])) vc->m_joystick[VC_DOWN]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_LEFT])) vc->m_joystick[VC_LEFT]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_RIGHT])) vc->m_joystick[VC_RIGHT]=true;
				if (k->key_pressed(m_keys_configuration2[KKEY_FIRE])) vc->m_button[0]=true;
			}
			
			// Prevent firing arrows because of the key press to start the game:
			if (m_state_cycle==0) {
				vc->m_old_button[0]=vc->m_button[0];
				vc->m_old_button[1]=vc->m_button[1];
			}
		}

		m_game->cycle(&m_vc_l,m_GLTM,m_SFXM,m_MusicM);
		
		// check if the players are dead:
		if (m_game_state==0 && m_game->get_map()->getNCharacters()==0) {
			m_game_state = 1;
			m_game_state_cycle = 0;
			m_MusicM->music_fadeout(0,50);
		}
	}
	
	if (m_game_state == 1) {
		m_game_state_cycle++;
		if (m_game_state_cycle>50) {
			return STATE_GAMEOVER;					
		}
	}


    return STATE_GAME;
}

void KnightmareApp::game_draw(void)
{
	glClearColor(0, 0, 0, 1);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

    m_game->draw(m_GLTM);
	
	// Draw the scoreboard:
	{
		int score_board_y = (m_state_cycle<32 ? m_state_cycle:32);
		score_board_y = 480 - score_board_y;

		glColor4f(0, 0, 0, 0.5);
		glBegin(GL_QUADS);
		glVertex3f(0, score_board_y, 0);
		glVertex3f(0, 480, 0);
		glVertex3f(640, 480, 0);
		glVertex3f(640, score_board_y, 0);

		glColor4f(0, 0, 0, 0.0);
		glVertex3f(0, score_board_y-4, 0);
		glColor4f(0, 0, 0, 0.5);
		glVertex3f(0, score_board_y, 0);
		glVertex3f(640, score_board_y, 0);
		glColor4f(0, 0, 0, 0.0);
		glVertex3f(640, score_board_y-4, 0);
		glEnd();	

		Binterface::print_center("SCORE",m_font16,128,score_board_y+16,1,1,1,1);
		Binterface::print_center("HISCORE",m_font16,256,score_board_y+16,1,1,1,1);
		Binterface::print_center("REST",m_font16,384,score_board_y+16,1,1,1,1);
		Binterface::print_center("STAGE",m_font16,512,score_board_y+16,1,1,1,1);

		char tmp[32];
		int i_tmp = 0;

		i_tmp = m_game->get_score();
		if (i_tmp<10) sprintf(tmp,"0%i",i_tmp);
				 else sprintf(tmp,"%i",i_tmp);
		Binterface::print_center(tmp,m_font16,128,score_board_y+32,1,1,1,1);

		if (m_n_players==1) {
			i_tmp = max(m_game->get_score(),get_1phiscore());
			if (i_tmp<10) sprintf(tmp,"0%i",i_tmp);
					 else sprintf(tmp,"%i",i_tmp);
		} else {
			i_tmp = max(m_game->get_score(),get_2phiscore());
			if (i_tmp<10) sprintf(tmp,"0%i",i_tmp);
					 else sprintf(tmp,"%i",i_tmp);
		}
		Binterface::print_center(tmp,m_font16,256,score_board_y+32,1,1,1,1);

		if (m_n_players==1) {
			if (m_n_lives_left_player1<10) sprintf(tmp,"0%i",m_n_lives_left_player1);
									  else sprintf(tmp,"%i",m_n_lives_left_player1);
		} else {
			sprintf(tmp,"%i/%i",m_n_lives_left_player1, m_n_lives_left_player2);
		}
		Binterface::print_center(tmp,m_font16,384,score_board_y+32,1,1,1,1);

		if (m_current_level<10) sprintf(tmp,"0%i",m_current_level);
						   else sprintf(tmp,"%i",m_current_level);
		Binterface::print_center(tmp,m_font16,512,score_board_y+32,1,1,1,1);
	}
	
	// pause:
	{
		float f = 0;
		if (m_game_paused_state>0) {
			f = m_game_paused_state/25.0f;
		} 
		if (m_game_paused_state<0) {
			f = m_game_paused_state/-25.0f;
		}
		
		if (m_game_paused_item_selected==0) {
			glColor4f(0, 0, 0, f*0.5);
		} else {
			glColor4f(0, 0, 0, 1-f*0.5);
		}

		if (m_game_paused_state!=0 || m_game_paused_item_selected!=0) {
			glBegin(GL_QUADS);
			glVertex3f(0, 0, 0);
			glVertex3f(0, 480, 0);
			glVertex3f(640, 480, 0);
			glVertex3f(640, 0, 0);
			glEnd();
			
			float blink = float(sin(m_state_cycle*0.25))*0.5f + 0.5f;
			Binterface::print_center("PAUSE",m_font32,320,120,1,1,1,f);
			if (m_game_paused_item_selected==0) {
				Binterface::print_center("RETURN TO GAME",m_font16,320,180,1,1,1,blink*f);
				Binterface::print_center("QUIT",m_font16,320,196,1,1,1,f);
			} else {
				Binterface::print_center("RETURN TO GAME",m_font16,320,180,1,1,1,f);
				Binterface::print_center("QUIT",m_font16,320,196,1,1,1,blink*f);		
			}
		}
	}	
	
	// fade out because player dead:
	if (m_game_state==1) {
		glColor4f(0, 0, 0, m_game_state_cycle/50.0);
		glBegin(GL_QUADS);
		glVertex3f(0, 0, 0);
		glVertex3f(0, 480, 0);
		glVertex3f(640, 480, 0);
		glVertex3f(640, 0, 0);
		glEnd();
	}
}

